package com.cpxt.biz.service;

import java.util.List;
import java.util.Map;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.cpxt.biz.domain.BizOrder;
import com.cpxt.biz.domain.vo.BizOrderVo;
import com.cpxt.biz.domain.vo.BizOrderVoCustomer;

/**
 * 订单Service接口
 * 
 * @author YIN
 * @date 2024-12-16
 */
public interface IBizOrderService 
{
    /**
     * 查询订单
     * 
     * @param id 订单主键
     * @return 订单
     */
    public BizOrder selectBizOrderById(Long id);

    /**
     * 查询订单列表
     * 
     * @param bizOrder 订单
     * @return 订单集合
     */
    public List<BizOrder> selectBizOrderList(BizOrder bizOrder);

    /**
     * 新增订单
     * 
     * @param bizOrder 订单
     * @return 结果
     */
    public int insertBizOrder(BizOrder bizOrder);

    /**
     * 修改订单
     * 
     * @param bizOrder 订单
     * @return 结果
     */
    public int updateBizOrder(BizOrder bizOrder);

    /**
     * 批量删除订单
     * 
     * @param ids 需要删除的订单主键集合
     * @return 结果
     */
    public int deleteBizOrderByIds(Long[] ids);

    /**
     * 删除订单信息
     * 
     * @param id 订单主键
     * @return 结果
     */
    public int deleteBizOrderById(Long id);

    Page<BizOrder> selectBizOrderPage(BizOrder bizOrder, int pageNum, int pageSize);

    String importOrder(List<BizOrderVo> orderList, boolean updateSupport);

    String importOrderCustomer(List<BizOrderVoCustomer> orderList, boolean updateSupport);

    Page<BizOrder> selectBizOrderPageByDriver(String orderStatus,int pageNum, int pageSize);

    public void insertBizOrderSub(Map map);

    public int goDelivery(Map map);
    public int arrived(Map map);

    Map<String,Long> getOrderStaticsByDriver();
}
